/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.genotype;

import java.util.Map.Entry;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.EncodingProviderModule;
import org.opt4j.core.Genotype;
import org.opt4j.core.genotype.CompositeGenotype;

/**
 * {@link CompositeGenotype} that additionally holds an {@link EncodingProviderModule} which
 * provides access to the {@link Genotype}s by Guice @Provides methods.
 * 
 * @author diewald
 */
public class ProvidingGenotype extends CompositeGenotype<Class<? extends Genotype>, Genotype> {

	/**
	 * Holds the reference to the {@link EncodingProviderModule} provided along this
	 * {@link CompositeGenotype}.
	 */
	private EncodingProviderModule<Genotype> providerModule;

	/**
	 * References the generator "singleton" that is used to construct
	 * {@link EncodingProviderModule}s.
	 */
	private GenotypeProviderGenerator providerGenerator;

	/** Constructor. */
	public ProvidingGenotype(
			CompositeGenotype<Class<? extends Genotype>, Genotype> compositeGenotype,
			GenotypeProviderGenerator providerGenerator) {
		for(Entry<Class<? extends Genotype>, Genotype> entry : compositeGenotype) {
			map.put(entry.getKey(), entry.getValue());
		}
		this.providerGenerator = providerGenerator;
	}

	/**
	 * Returns the {@link EncodingProviderModule} that is passed along this
	 * {@link CompositeGenotype}.
	 */
	public EncodingProviderModule<Genotype> getProviderModule() {
		if(providerModule == null) {
			this.providerModule = providerGenerator.generateInstanceProviders(values());
		}
		return providerModule;
	}

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	public ProvidingGenotype newInstance() {
		return new ProvidingGenotype(this, providerGenerator);
	}
}
