/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.evaluator.constraint;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.module.EvaluatorModule;
import org.fortiss.af3.exploration.model.ExplorationConstraint;
import org.opt4j.core.problem.Evaluator;

/**
 * Base class for {@link Evaluator}s of constraints.
 * 
 * @author diewald
 */
public abstract class ConstraintEvaluatorBase<R, T extends ExplorationConstraint<R>, P extends Phenotype>
		extends EvaluatorModule<R, T, P> {

	/**
	 * Defines the acceptable numeric imprecision, i.e. the allowed violation of constraints during
	 * the validation phase.
	 */
	protected final static double NUMERIC_IMPRECISION = 10E-9;

	/** Prototype constructor. */
	public ConstraintEvaluatorBase(Class<? extends T> explorationTargetType) {
		super(explorationTargetType);
	}

	/** Creates an evaluator for a given exploration goal. */
	public ConstraintEvaluatorBase(T explorationTarget) {
		super(explorationTarget);
	}
}
