/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.StrictTTDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.solution.StrictTTSchedule;

import com.google.inject.Injector;

/**
 * Decodes a given mapping of task and a mapping of messages (routes) into a time-triggered
 * schedule. First a EDF task scheduling heuristic is applied, before a TDMA message scheduler
 * inserts the slots required for communication. The latter step "stretches" the task schedule.
 * 
 * @author diewald
 */
public class TTScheduleExploration extends ExplorationFeature {

	/** Constructor. */
	public TTScheduleExploration() {
		super("Decoding mappings -> schedules");

		addProblem(StrictTTSchedule.class, new StrictTTDecoder());
	}

	/** {@inheritDoc} */
	@Override
	public boolean isApplicableToInputs(Injector dseInjector) {
		return true;
	}
}
