/*-------------------------------------------------------------------------+
| Copyright 2016 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.service.module;

import java.util.Collection;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.fortiss.af3.exploration.alg.service.IExplorationExtension;
import org.fortiss.af3.exploration.model.ExplorationTarget;
import org.opt4j.core.Value;
import org.opt4j.core.problem.Evaluator;

import com.google.common.collect.ClassToInstanceMap;
import com.google.inject.Module;

/**
 * Interface to be implemented by {@link IExplorationExtension}s that perform evaluations of
 * individuals (= candidate solutions). These are generated during the execution of the DSE.
 * 
 * @author diewald
 */
// TODO(AD): Extend the documentation: Use parameters! We have an interface here.
public interface IExplorationTargetEvaluator<P, RT, S extends ExplorationTarget<RT>>
		extends Module {

	/** Evaluates a phenotype w.r.t. a given objective dimension. */
	public Value<?> evaluateGoal(P phenotype, ClassToInstanceMap<EObject> transformedModels)
			throws Exception;

	/** Validate if objective has been correctly specified. */
	public boolean validateInputs() throws Exception;

	/**
	 * Returns the set of input types (classes) which are required for the enclosed
	 * {@link Evaluator}.
	 */
	Collection<Class<?>> getRequiredInputTypes();

	/** Returns the phenotype type for which this evaluator is registered. */
	// Only needed until the automatic model transformation is finished
	Class<?> getPrimaryInputType();

	/**
	 * Creates an instance of {@code this} {@link IExplorationTargetEvaluator}. The
	 * {@code systemModelAdapter} and the {@code explorationTarget} are passed from the DSE
	 * framework.
	 * 
	 * @param systemModelAdapter
	 *            Internal representation of the base models.
	 * @param explorationTarget
	 *            Specifies the concrete exploration target including the parameters.
	 * @return A new instance of {@code this} {@link IExplorationTargetEvaluator}.
	 * @throws InstantiationException
	 *             If the instantiation of a new {@link IExplorationTargetEvaluator} fails.
	 * @throws Exception
	 *             if the input validation fails.
	 */
	IExplorationTargetEvaluator<P, RT, S> createEvaluator(
			SystemModelAdapter<?, ?, ?, ?, ?, ?, ?> systemModelAdapter, S explorationTarget)
			throws InstantiationException, Exception;
}
