/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.servicemodule.transform.expspec.constraint;

import static org.fortiss.af3.exploration.alg.util.DSMLUtils.getDirectAllocationExpression;
import static org.fortiss.af3.exploration.alg.util.DesignSpaceExplorationModelElementFactory.createAssignmentConstraint;

import java.util.Collection;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.mappers.ExecUnitMapper;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.mappers.TaskMapper;
import org.fortiss.af3.exploration.alg.service.module.base.ExpSpecElementTransformatorBase;
import org.fortiss.af3.exploration.dsl_v2.model.booleanp.allocation.ILocationExpression;
import org.fortiss.af3.exploration.dsl_v2.model.expression.Set;
import org.fortiss.af3.exploration.model.ExplorationConstraint;
import org.fortiss.af3.exploration.moea.model.constraints.AssignmentConstraint;
import org.fortiss.af3.exploration.moea.model.constraints.InternalConstraint;
import org.fortiss.tooling.base.model.element.IModelElement;
import org.fortiss.tooling.kernel.introspection.IIntrospectionDetailsItem;
import org.fortiss.tooling.kernel.introspection.IIntrospectionItem;

import com.google.inject.Injector;
import com.google.inject.Key;
import com.google.inject.TypeLiteral;

/**
 * Converts an allocation pattern {@link ExplorationConstraint} to an {@link InternalConstraint}.
 * 
 * @author diewald
 */
public final class AllocationPatternConverter
		extends ExpSpecElementTransformatorBase<ExplorationConstraint<?>, InternalConstraint<?>> {

	/** {@inheritDoc} */
	@Override
	public InternalConstraint<?> transform(Injector mainInjector,
			ExplorationConstraint<?> constraint) {
		TaskMapper<EObject> taskMapper =
				mainInjector.getInstance(Key.get(new TypeLiteral<TaskMapper<EObject>>() { // ...
				}));
		ExecUnitMapper<EObject> execUnitMapper =
				mainInjector.getInstance(Key.get(new TypeLiteral<ExecUnitMapper<EObject>>() { // ...
				}));

		ILocationExpression locExpression = getDirectAllocationExpression(constraint);
		// TODO(AD): The mappers will not be needed any more when removing the adapters. Then, the
		// code can be also more generic by the set types.
		if(locExpression == null) {
			// No allocation pattern. Exit early.
			return null;
		}
		AssignmentConstraint assignConstr = createAssignmentConstraint(locExpression);

		Set<IModelElement> requesterSet = locExpression.getLeft().getSetReference();
		for(IModelElement me : requesterSet.getEntries()) {
			assignConstr.getSources().add(taskMapper.getAdapter(me));
		}
		Set<IModelElement> resourceSet = locExpression.getRight().getSetReference();
		for(IModelElement me : resourceSet.getEntries()) {
			assignConstr.getTargets().add(execUnitMapper.getAdapter(me));
		}
		return assignConstr;
	}

	/** {@inheritDoc} */
	@Override
	public String getIntrospectionLabel() {
		// TODO Auto-generated method stub
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public boolean showInIntrospectionNavigation() {
		// TODO Auto-generated method stub
		return false;
	}

	/** {@inheritDoc} */
	@Override
	public Collection<IIntrospectionItem> getIntrospectionItems() {
		// TODO Auto-generated method stub
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public IIntrospectionDetailsItem getDetailsItem() {
		// TODO Auto-generated method stub
		return null;
	}
}
